package javax.swing.plaf.metal;

import javax.swing.KeyStroke;
import java.awt.event.KeyEvent;
import javax.swing.AbstractButton;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.FontMetrics;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;
import java.awt.Rectangle;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.JToolTip;
import java.awt.Font;
import javax.swing.plaf.basic.BasicToolTipUI;

public class MetalToolTipUI extends BasicToolTipUI
{
    static MetalToolTipUI sharedInstance;
    private Font smallFont;
    private JToolTip tip;
    public static final int padSpaceBetweenStrings = 12;
    private String acceleratorDelimiter;
    
    public MetalToolTipUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return MetalToolTipUI.sharedInstance;
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        this.tip = (JToolTip)c;
        Font font = c.getFont();
        this.smallFont = new Font(font.getName(), font.getStyle(), font.getSize() - 2);
        this.acceleratorDelimiter = UIManager.getString("MenuItem.acceleratorDelimiter");
        if (this.acceleratorDelimiter == null) {
            this.acceleratorDelimiter = "-";
        }
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        this.tip = null;
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        JToolTip toolTip = (JToolTip)component;
        Font font = component.getFont();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(component, graphics, font);
        Dimension size = component.getSize();
        graphics.setColor(component.getForeground());
        String tipText = toolTip.getTipText();
        if (tipText == null) {
            tipText = "";
        }
        String acceleratorString = this.getAcceleratorString(toolTip);
        int calcAccelSpacing = this.calcAccelSpacing(component, SwingUtilities2.getFontMetrics(component, graphics, this.smallFont), acceleratorString);
        Insets insets = toolTip.getInsets();
        Rectangle o = new Rectangle(insets.left + 3, insets.top, size.width - (insets.left + insets.right) - 6 - calcAccelSpacing, size.height - (insets.top + insets.bottom));
        View view = (View)component.getClientProperty("html");
        int n;
        if (view != null) {
            view.paint(graphics, o);
            n = BasicHTML.getHTMLBaseline(view, o.width, o.height);
        }
        else {
            graphics.setFont(font);
            SwingUtilities2.drawString(toolTip, graphics, tipText, o.x, o.y + fontMetrics.getAscent());
            n = fontMetrics.getAscent();
        }
        if (!acceleratorString.equals("")) {
            graphics.setFont(this.smallFont);
            graphics.setColor(MetalLookAndFeel.getPrimaryControlDarkShadow());
            SwingUtilities2.drawString(toolTip, graphics, acceleratorString, toolTip.getWidth() - 1 - insets.right - calcAccelSpacing + 12 - 3, o.y + n);
        }
    }
    
    private int calcAccelSpacing(JComponent c, FontMetrics fm, String string) {
        return string.equals("") ? 0 : (12 + SwingUtilities2.stringWidth(c, fm, string));
    }
    
    @Override
    public Dimension getPreferredSize(JComponent c) {
        Dimension preferredSize = super.getPreferredSize(c);
        String acceleratorString = this.getAcceleratorString((JToolTip)c);
        if (!acceleratorString.equals("")) {
            preferredSize.width += this.calcAccelSpacing(c, c.getFontMetrics(this.smallFont), acceleratorString);
            return preferredSize;
        }
        return preferredSize;
    }
    
    protected boolean isAcceleratorHidden() {
        Boolean b = (Boolean)UIManager.get("ToolTip.hideAccelerator");
        return b != null && (boolean)b;
    }
    
    private String getAcceleratorString(JToolTip tip) {
        this.tip = tip;
        String acceleratorString = this.getAcceleratorString();
        this.tip = null;
        return acceleratorString;
    }
    
    public String getAcceleratorString() {
        if (this.tip == null || this.isAcceleratorHidden()) {
            return "";
        }
        JComponent component = this.tip.getComponent();
        if (!(component instanceof AbstractButton)) {
            return "";
        }
        KeyStroke[] keys = component.getInputMap(2).keys();
        if (keys == null) {
            return "";
        }
        String string = "";
        int n = 0;
        if (n < keys.length) {
            string = KeyEvent.getKeyModifiersText(keys[n].getModifiers()) + this.acceleratorDelimiter + KeyEvent.getKeyText(keys[n].getKeyCode());
            return string;
        }
        return string;
    }
    
    static {
        MetalToolTipUI.sharedInstance = new MetalToolTipUI();
    }
}
